﻿'//////////////////////////////////////////////////////////////////////////////////////////////////
' 
'   ACT距離計算サービス サンプルプログラム
'   計算用道路データ選択フォームクラス
'
'   Copyrights (c) 2010-2011 Advanced Core Technologies, Inc. All rights reserved.
'
'  2010/09/16 Rev.2.1.1.0 ISHI 新規作成
'  2010/12/03 Rev.2.1.2.0 ISHI SSL設定追加
'  2011/03/30 Rev.2.2.0.0 ISHI Userクラス対応
'
'//////////////////////////////////////////////////////////////////////////////////////////////////

''' <summary>
''' 計算用道路データ選択フォームクラス
''' </summary>
''' <remarks></remarks>
Public Class RoadDataForm

    '----------------------------------------------------------------------------------------------
    '	Private Fields
    '----------------------------------------------------------------------------------------------
#Region "Private Fields"

    ''' <summary>
    ''' 計算用道路データ情報クラス
    ''' </summary>
    Private nwInfo_ As DCAdmin.NWInfo

    ''' <summary>
    ''' 現在のページ番号
    ''' </summary>
    Private pageIndex_ As Int32 = 0

    ''' <summary>
    ''' 計算用道路データ情報オブジェクト配列
    ''' </summary>
    Private nwInfos_ As DCAdmin.NWInfo()

    ''' <summary>
    ''' ユーザオブジェクト
    ''' </summary>
    Private user_ As User = Nothing

#End Region

    '----------------------------------------------------------------------------------------------
    '	Properties
    '----------------------------------------------------------------------------------------------
#Region "Properties"

    ''' <summary>
    ''' ユーザオブジェクトを取得する
    ''' </summary>
    ''' <value></value>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Public ReadOnly Property User() As User
        Get
            Return Me.user_
        End Get
    End Property

#End Region

    '----------------------------------------------------------------------------------------------
    '	Private Methods
    '----------------------------------------------------------------------------------------------
#Region "Private Methods"

    ''' <summary>
    ''' 指定ページを表示する
    ''' </summary>
    ''' <param name="pageIndex">表示するページ番号</param>
    Private Sub ShowPage(ByVal pageIndex As Int32)
        Select Case pageIndex
            Case 0
                ' 1ページ目
                ' パネル内の初期化
                InitializeServerPage()

                ' コントロールの表示設定変更
                buttonFinal.Enabled = False
                buttonPrev.Enabled = False
                buttonNext.Enabled = True
                panelFooter.Top = panelServer.Bounds.Bottom + 1
                Me.ClientSize = New Size(Me.ClientSize.Width, panelFooter.Bounds.Bottom)
                panelServer.Visible = True
                panelRoad.Visible = False
                textUserID.Focus()
                Me.AcceptButton = buttonNext

                ' ページ番号変更
                pageIndex_ = pageIndex
            Case 1
                ' 2ページ目
                ' パネル内の初期化
                InitializeRoadPage()

                ' コントロールの表示設定変更
                buttonFinal.Enabled = False
                buttonPrev.Enabled = True
                buttonNext.Enabled = False
                panelRoad.Left = panelServer.Left
                panelRoad.Top = panelServer.Top
                panelFooter.Top = panelRoad.Bounds.Bottom + 1
                Me.ClientSize = New Size(Me.ClientSize.Width, panelFooter.Bounds.Bottom)
                panelRoad.Visible = True
                panelServer.Visible = False
                ListViewRoad.Focus()
                Me.AcceptButton = buttonFinal
                ChangeFinalButtonState()

                ' ページ番号変更
                pageIndex_ = pageIndex
            Case Else
        End Select
        ' キャプション変更
        Me.Text = String.Format("計算用道路データ選択ウィザード - {0} / 2", pageIndex_ + 1)
    End Sub

    ''' <summary>
    ''' サーバーページの初期化
    ''' </summary>
    Private Sub InitializeServerPage()
        '	UIにデータをロード
        textUserID.Text = Me.User.UserID        '  ユーザID
        textDCAdminURL.Text = Me.User.AdminUrl  '  管理サーバURL
        checkUseSsl.Checked = Me.User.SslMode   '   SSLモード
    End Sub

    ''' <summary>
    ''' 計算用道路選択ページの初期化
    ''' </summary>
    Private Sub InitializeRoadPage()
        ' 計算用道路データリストビュークリア
        ListViewRoad.Items.Clear()

        '	配列の要素数取得 
        Dim arrayLength As Int32 = nwInfos_.Length

        '	ListView に 計算用道路データ情報オブジェクトの各メンバを表示する
        For i As Int32 = 0 To arrayLength - 1
            ListViewRoad.Items.Add(nwInfos_(i).Note)                                                ' 計算用道路データ名

            'NWIDに応じて対応輸送機関に○×を表示する
            Dim flag As Int32 = Convert.ToInt32(nwInfos_(i).NWID.ToString().Substring(5, 3), 16)    ' フラグ
            ' フラグの意味　自動車,歩行者,航空機,フェリー,旅客鉄道,貨物鉄道,内航船
            Dim flagMeans() As Int32 = {Act.Dcws.NwidClassA.Car, _
                                        Act.Dcws.NwidClassA.Walk, _
                                        Act.Dcws.NwidClassA.Airway, _
                                        Act.Dcws.NwidClassA.Ferry, _
                                        Act.Dcws.NwidClassA.Railroad, _
                                        Act.Dcws.NwidClassA.RailroadFreight, _
                                        Act.Dcws.NwidClassA.Ship}
            Dim subItems(6) As String                                                               ' サブアイテム文字列の配列
            For j As Int32 = 0 To 6
                subItems(j) = If((flag And flagMeans(j)) <> 0, "○", "×")
            Next
            ListViewRoad.Items(i).SubItems.AddRange(subItems)

            ListViewRoad.Items(i).SubItems.Add(nwInfos_(i).NWID.ToString())                         ' NWID
        Next

        ' リストアイテムの選択
        If Not nwInfo_.Note Is Nothing Then
            '	現在設定されている計算用道路データがある場合
            For i As Int32 = 0 To arrayLength - 1
                '	設定されている計算用道路データのアイテムか
                If String.Compare(ListViewRoad.Items(i).Text, nwInfo_.Note) = 0 Then
                    '	そのアイテムを選択
                    ListViewRoad.Items(i).Selected = True
                    ListViewRoad.Items(i).Focused = True
                    Exit For    '	ループを抜ける
                End If
            Next
        Else
            ' 現在設定されている計算用道路データがない場合
            If ListViewRoad.Items.Count > 0 Then
                ' 先頭のアイテムを選択する
                ListViewRoad.Items(0).Selected = True
            End If
        End If
    End Sub

    ''' <summary>
    ''' 完了ボタンの状態を変更する
    ''' </summary>
    Private Sub ChangeFinalButtonState()
        If panelRoad.Visible AndAlso ListViewRoad.SelectedIndices.Count > 0 Then
            ' 選択アイテムがある場合完了ボタンを有効にする
            buttonFinal.Enabled = True
        Else
            ' 選択アイテムがない場合完了ボタンを無効にする
            buttonFinal.Enabled = False
        End If
    End Sub

#End Region

    '----------------------------------------------------------------------------------------------
    '	Public Methods
    '----------------------------------------------------------------------------------------------
#Region "Public Methods"

    ''' <summary>
    ''' コンストラクタ
    ''' </summary>
    ''' <param name="user">ユーザオブジェクト</param>
    Public Sub New(ByVal user As User)
        InitializeComponent()
        Size = New Size(533, 235)

        '	環境設定
        Me.user_ = user
        nwInfo_ = New DCAdmin.NWInfo()
    End Sub

#End Region

    '----------------------------------------------------------------------------------------------
    '	Event Handlers
    '----------------------------------------------------------------------------------------------
#Region "Event Handlers"

    ''' <summary>
    ''' フォームロード時
    ''' </summary>
    ''' <param name="sender"></param>
    ''' <param name="e"></param>
    ''' <remarks></remarks>
    Private Sub RoadDataForm_Load(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MyBase.Load
        ' サーバーページ初期化
        InitializeServerPage()
        ' ページ表示
        pageIndex_ = 0
        ShowPage(pageIndex_)

    End Sub

    ''' <summary>
    ''' 次へボタンクリック時
    ''' </summary>
    ''' <param name="sender"></param>
    ''' <param name="e"></param>
    ''' <remarks></remarks>
    Private Sub buttonNext_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles buttonNext.Click
        If pageIndex_ = 0 Then
            '  1ページ目の場合
            Dim adminUrl As String   '  管理サーバURL
            '	環境設定
            Try
                Dim testUri As Uri = New Uri(textDCAdminURL.Text)

                ' http,https以外のプロトコルは中止
                If testUri.Scheme <> "http" And testUri.Scheme <> "https" Then
                    MessageBox.Show(Me, "管理サーバ URLを 「http://」または「https://」 から入力してください。", "エラー", _
                                     MessageBoxButtons.OK, MessageBoxIcon.Error)
                    Return
                End If

                ' https入力　かつ　SSLを使用するチェックボックスが未チェックの場合、確認する
                If testUri.Scheme = "https" AndAlso Not checkUseSsl.Checked Then
                    If (MessageBox.Show(Me, "管理サーバURLにhttpsプロトコルが入力されていますが、" + _
                                        "SSLを使用するにチェックがありません。SSLを使用しますか？", _
                                        "SSL使用確認", MessageBoxButtons.YesNo, MessageBoxIcon.Question) = DialogResult.Yes) Then
                        checkUseSsl.Checked = True
                    End If
                End If

                '	管理サーバ URLを設定する
                adminUrl = Utilities.ChangeProtocol(testUri.ToString(), checkUseSsl.Checked)
            Catch ex As Exception
                MessageBox.Show(Me, "管理サーバ URLを http: から入力してください。" + vbCrLf + ex.Message, _
                                "エラー", MessageBoxButtons.OK, MessageBoxIcon.Error)
                Return
            End Try

            ' ユーザオブジェクト生成
            user_ = New User(textUserID.Text, textUUID.Text, adminUrl, checkUseSsl.Checked)
            '  インスタンスの作成
            Dim dcAdminServer As DCAdmin.ACTDCADMIN = New DCAdmin.ACTDCADMIN()  '  管理サーバオブジェクト
            Dim dcAdminEnviron As DCAdmin.Environ = Me.User.GetAdminEnviron()   '  管理サーバ環境設定オブジェクト
            dcAdminServer.Url = Me.User.AdminUrl                                '  管理サーバ URLを設定
            Try
                '  ユーザ情報取得
                Dim userInfo As DCAdmin.UserInformation2 = New SampleMG.DCAdmin.UserInformation2()
                If Not dcAdminServer.GetUserInformation2(dcAdminEnviron, userInfo) Then
                    MessageBox.Show(Me, "ユーザ情報の取得に失敗しました。" + Environment.NewLine + _
                                    "正しいユーザID、パスワード、管理サーバURLを入力してください。", _
                                    "管理サーバ", MessageBoxButtons.OK, MessageBoxIcon.Error)
                    Return
                End If
                User.UserInfo = userInfo

                nwInfos_ = Nothing
                '  使用できる計算用道路データ情報を取得
                If Not dcAdminServer.GetUsableNWInfos(dcAdminEnviron, nwInfos_) Then
                    MessageBox.Show(Me, "計算用道路データ情報の取得に失敗しました。" + vbCrLf + _
                                    "正しいユーザID、パスワード、管理サーバURLを入力してください。", _
                                    "管理サーバ", MessageBoxButtons.OK, MessageBoxIcon.Error)
                    Return
                End If
            Catch ex As Exception
                MessageBox.Show(Me, ex.Message, "管理サーバ", MessageBoxButtons.OK, MessageBoxIcon.Error)
                Return
            End Try

            ' 次ページを表示
            ShowPage(pageIndex_ + 1)
        End If

    End Sub

    ''' <summary>
    ''' 完了ボタンクリック時
    ''' </summary>
    ''' <param name="sender"></param>
    ''' <param name="e"></param>
    ''' <remarks></remarks>
    Private Sub buttonFinal_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles buttonFinal.Click
        If pageIndex_ = 1 Then
            ' 2ページ目
            If ListViewRoad.SelectedIndices.Count = 1 Then
                ' 計算用道路が選択されている場合
                Me.User.NWInfo = Me.nwInfos_(ListViewRoad.SelectedIndices(0))
            End If
        End If
    End Sub

    ''' <summary>
    ''' 前へボタンクリック時
    ''' </summary>
    ''' <param name="sender"></param>
    ''' <param name="e"></param>
    Private Sub buttonPrev_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles buttonPrev.Click
        If pageIndex_ > 0 Then
            ' 前ページを表示
            ShowPage(pageIndex_ - 1)
        End If
    End Sub

    ''' <summary>
    ''' 計算用道路リストビューの選択アイテムインデックスが変更されたとき
    ''' </summary>
    ''' <param name="sender"></param>
    ''' <param name="e"></param>
    Private Sub ListViewRoad_SelectedIndexChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ListViewRoad.SelectedIndexChanged
        ' 完了ボタンの状態を変更する
        ChangeFinalButtonState()
    End Sub

#End Region

End Class