﻿'//////////////////////////////////////////////////////////////////////////////////////////////////
' 
'   ACT距離計算サービス サンプルプログラム
'   住所検索フォームクラス
'
'   Copyrights (c) 2010-2011 Advanced Core Technologies, Inc. All rights reserved.
'
'   2010/12/03 Rev.2.1.2.0 ISHI 初版作成
'   2011/08/25 Rev.2.2.0.0 ISHI 住所検索サービス新関数対応、Userクラス対応
'
'//////////////////////////////////////////////////////////////////////////////////////////////////
Imports Act.Dcws
Imports System.Text.RegularExpressions

''' <summary>
''' 検索モード
''' </summary>
''' <remarks></remarks>
Public Enum SearchMode As Integer
    ''' <summary>
    ''' 住所
    ''' </summary>
    ''' <remarks></remarks>
    Address = 0
    ''' <summary>
    ''' 郵便番号
    ''' </summary>
    ''' <remarks></remarks>
    Zip = 1
End Enum


''' <summary>
''' 住所検索フォーム
''' </summary>
''' <remarks></remarks>
Public Class SearchAddressForm

    '----------------------------------------------------------------------------------------------
    '	Class
    '----------------------------------------------------------------------------------------------
#Region "Class"

    ''' <summary>
    ''' 絞り込み検索用住所レベル別一覧データクラス
    ''' </summary>
    ''' <remarks></remarks>
    Class EnumAddressData
        '------------------------------------------------------------------------------------------
        '	Private Fields
        '------------------------------------------------------------------------------------------
#Region "Private Fields"

        ''' <summary>
        ''' 選択されている住所アイテム
        ''' </summary>
        ''' <remarks></remarks>
        Private currentItem_ As GCService.AddressItem

        ''' <summary>
        ''' 下位住所列挙で取得した住所アイテム配列
        ''' </summary>
        ''' <remarks></remarks>
        Private enumItems_() As GCService.AddressItem

        ''' <summary>
        ''' 対応するリストビュー
        ''' </summary>
        ''' <remarks></remarks>
        Private listView_ As ListView

        ''' <summary>
        ''' リストビューに表示する住所アイテムリスト
        ''' </summary>
        ''' <remarks></remarks>
        Private showItems_ As New List(Of GCService.AddressItem)

        ''' <summary>
        ''' フィルター用テキストボックス
        ''' </summary>
        ''' <remarks></remarks>
        Private filterTextBox_ As TextBox

#End Region

        '------------------------------------------------------------------------------------------
        '	Properties
        '------------------------------------------------------------------------------------------
#Region "Properties"

        ''' <summary>
        ''' 現在選択されている住所アイテムを取得・設定する
        ''' </summary>
        ''' <value></value>
        ''' <returns></returns>
        ''' <remarks></remarks>
        Public Property CurrentItem() As GCService.AddressItem
            Get
                Return Me.currentItem_
            End Get
            Private Set(ByVal value As GCService.AddressItem)
                Me.currentItem_ = value
            End Set
        End Property

        ''' <summary>
        ''' 現在列挙中の住所アイテム配列を取得・設定する
        ''' </summary>
        ''' <value></value>
        ''' <returns></returns>
        ''' <remarks></remarks>
        Public Property EnumItems() As GCService.AddressItem()
            Get
                Return Me.enumItems_
            End Get
            Private Set(ByVal value As GCService.AddressItem())
                Me.enumItems_ = value
            End Set
        End Property

        ''' <summary>
        ''' 対応するリストビューを取得・設定する
        ''' </summary>
        ''' <value></value>
        ''' <returns></returns>
        ''' <remarks></remarks>
        Public Property ListView() As ListView
            Get
                Return Me.listView_
            End Get
            Private Set(ByVal value As ListView)
                Me.listView_ = value
            End Set
        End Property

        ''' <summary>
        ''' リストビューに表示する住所アイテムリストを取得・設定する
        ''' </summary>
        ''' <value></value>
        ''' <returns></returns>
        ''' <remarks></remarks>
        Public Property ShowItems() As List(Of GCService.AddressItem)
            Get
                Return Me.showItems_
            End Get
            Private Set(ByVal value As List(Of GCService.AddressItem))
                Me.showItems_ = value
            End Set
        End Property

        ''' <summary>
        ''' フィルター用テキストボックスを取得・設定する
        ''' </summary>
        ''' <value></value>
        ''' <returns></returns>
        ''' <remarks></remarks>
        Public Property FilterTextBox() As TextBox
            Get
                Return Me.filterTextBox_
            End Get
            Private Set(ByVal value As TextBox)
                Me.filterTextBox_ = value
            End Set
        End Property

#End Region

        '------------------------------------------------------------------------------------------
        '	Public Methods
        '------------------------------------------------------------------------------------------
#Region "Public Methods"

        ''' <summary>
        ''' コンストラクタ
        ''' </summary>
        ''' <remarks></remarks>
        ''' <param name="listView">リストビュー</param>
        ''' <param name="filterTextBox">フィルター用テキストボックス</param>
        Sub New(ByVal listView As ListView, ByVal filterTextBox As TextBox)
            Me.ListView = listView
            Me.FilterTextBox = filterTextBox
        End Sub

        ''' <summary>
        ''' 全てのフィールドをクリアする
        ''' </summary>
        ''' <remarks></remarks>
        Public Sub ClearAll()
            listView_.Items.Clear()
            currentItem_ = Nothing
            enumItems_ = New GCService.AddressItem() {}
            showItems_.Clear()
            If Not filterTextBox_ Is Nothing Then
                filterTextBox_.Text = ""
            End If
        End Sub

        ''' <summary>
        ''' リストビュー関連フィールドをクリアする
        ''' </summary>
        ''' <remarks></remarks>
        Public Sub ClearList()
            listView_.Items.Clear()
            enumItems_ = New GCService.AddressItem() {}
            showItems_.Clear()
            If Not filterTextBox_ Is Nothing Then
                filterTextBox_.Text = ""
            End If
        End Sub

        ''' <summary>
        ''' 現在選択されている住所アイテムをクリアする
        ''' </summary>
        ''' <remarks></remarks>
        Public Sub ClearCurrent()
            currentItem_ = Nothing
        End Sub

        ''' <summary>
        ''' ListViewの再描画
        ''' </summary>
        ''' <param name="topItemIndex">リストビューの最上位に表示されるアイテムのインデックス</param>
        ''' <remarks></remarks>
        Public Sub RedrawListView(ByVal topItemIndex As Int32)
            Dim selectIndex As Int32 = -1   ' 選択アイテムのインデックス

            ' クリア
            ListView.Items.Clear()

            ' リストビューにアイテムを追加
            If Not IsNothing(ShowItems) Then
                ListView.BeginUpdate()
                Dim listItem As ListViewItem
                For Each item As GCService.AddressItem In ShowItems
                    If Not IsNothing(CurrentItem) AndAlso item.AddressCode = CurrentItem.AddressCode Then
                        ' 選択されたアイテムの場合、目印を付ける
                        listItem = ListView.Items.Add("●" & item.Address)
                        listItem.Tag = item.AddressCode
                        listItem.ToolTipText = item.Address
                        selectIndex = listItem.Index
                    Else
                        listItem = ListView.Items.Add("   " & item.Address)
                        listItem.Tag = item.AddressCode
                        listItem.ToolTipText = item.Address
                    End If
                Next
                ListView.EndUpdate()
            End If

            ' カラムサイズ設定
            ListView.AutoResizeColumn(0, ColumnHeaderAutoResizeStyle.ColumnContent)
            If ListView.Columns(0).Width < ListView.ClientRectangle.Width Then
                'Selectの反転を横幅いっぱいにするため不足分を足す
                ListView.Columns(0).Width = ListView.ClientRectangle.Width
            End If

            ' 表示設定
            If Not IsNothing(ShowItems) AndAlso ListView.Items.Count > 0 Then
                ListView.BeginUpdate()
                ' TopItem設定
                If topItemIndex >= 0 Then
                    ListView.Items(ListView.Items.Count - 1).EnsureVisible()
                    ListView.Items(topItemIndex).EnsureVisible()
                Else
                    If selectIndex >= 0 Then
                        ListView.Items(selectIndex).EnsureVisible()
                    End If
                End If
                ' フォーカス設定
                If selectIndex >= 0 Then
                    ListView.Items(selectIndex).Focused = True
                End If
                ListView.EndUpdate()
            End If
        End Sub

        ''' <summary>
        ''' リストビューフィルター変更に伴う更新をする
        ''' </summary>
        ''' <remarks></remarks>
        Public Sub UpdateListViewFilter()
            ' リストビューに表示されているアイテムをクリア
            ListView.Items.Clear()

            ShowItems.Clear()

            ' フィルター文字列作成　全角に変換
            Dim filterText As String = Strings.StrConv(FilterTextBox.Text, VbStrConv.Wide)

            ' フィルター文字列を含むアイテムを抽出
            For Each item As GCService.AddressItem In EnumItems
                If item.Address.IndexOf(filterText) <> -1 Then
                    ShowItems.Add(item)
                End If
            Next

            ' 抽出したアイテムを表示
            RedrawListView(-1)
        End Sub

        ''' <summary>
        ''' 住所アイテム配列をShowItemsにコピーする
        ''' </summary>
        ''' <param name="srcItems">コピー元住所アイテム配列</param>
        ''' <remarks></remarks>
        Public Sub CopyToShowItems(ByVal srcItems As GCService.AddressItem())
            Dim item As GCService.AddressItem

            EnumItems = Nothing
            If (Not srcItems Is Nothing) Then
                EnumItems = New GCService.AddressItem(srcItems.Length - 1) {}
                For i As Int32 = 0 To srcItems.Length - 1
                    item = srcItems(i)
                    EnumItems(i) = New GCService.AddressItem()
                    EnumItems(i).AddressLevel = item.AddressLevel
                    EnumItems(i).AddressCode = item.AddressCode
                    EnumItems(i).Address = item.Address
                    EnumItems(i).Zip = item.Zip
                    EnumItems(i).Lon = item.Lon
                    EnumItems(i).Lat = item.Lat
                    EnumItems(i).Kana = item.Kana
                    EnumItems(i).KataKana = item.KataKana
                    EnumItems(i).UpperAddressLevel = item.UpperAddressLevel
                    EnumItems(i).LowerAddressLevel = item.LowerAddressLevel
                Next
            End If

            ShowItems.AddRange(EnumItems)
        End Sub

        ''' <summary>
        ''' 住所アイテムをCurrentItemにコピーする
        ''' </summary>
        ''' <param name="srcItem">コピー元住所アイテム</param>
        ''' <returns>成功時はTrue。失敗時はFalse。</returns>
        ''' <remarks></remarks>
        Public Function CopyToCurrentItem(ByVal srcItem As GCService.AddressItem) As Boolean
            Dim success As Boolean = True
            Try
                If Not srcItem Is Nothing Then
                    If CurrentItem Is Nothing Then
                        CurrentItem = New GCService.AddressItem()
                    End If
                    CurrentItem.AddressLevel = srcItem.AddressLevel
                    CurrentItem.AddressCode = srcItem.AddressCode
                    CurrentItem.Address = srcItem.Address
                    CurrentItem.Zip = srcItem.Zip
                    CurrentItem.Lon = srcItem.Lon
                    CurrentItem.Lat = srcItem.Lat
                    CurrentItem.Kana = srcItem.Kana
                    CurrentItem.KataKana = srcItem.KataKana
                    CurrentItem.UpperAddressLevel = srcItem.UpperAddressLevel
                    CurrentItem.LowerAddressLevel = srcItem.LowerAddressLevel
                Else
                    success = False
                End If
            Catch
                success = False
            End Try

            Return success
        End Function

        ''' <summary>
        ''' CurrentItemをShowItemsのindex番目の住所アイテムに変更する
        ''' </summary>
        ''' <param name="index">CurrentItemにする住所アイテムのインデックス番号</param>
        Public Sub SelectCurrentItem(ByVal index As Int32)
            CurrentItem = ShowItems(index)
        End Sub

        ''' <summary>
        ''' CurrentItemをShowItemsの中でaddressCodeと同じ住所コードの住所アイテムに変更する
        ''' </summary>
        ''' <param name="addressCode">住所コード</param>
        Public Sub SelectCurrentItem(ByVal addressCode As String)
            For Each addressItem As GCService.AddressItem In Me.EnumItems
                If addressCode = addressItem.AddressCode Then
                    CopyToCurrentItem(addressItem)
                    Return
                End If
            Next
        End Sub

#End Region

    End Class

#End Region

    '----------------------------------------------------------------------------------------------
    '	Constants
    '----------------------------------------------------------------------------------------------
#Region "Constants"

    ''' <summary>
    ''' 住所レベル別コード長配列
    ''' </summary>
    ''' <remarks>インデックス番号：０＝都道府県、１＝市区町村、２＝大字・丁目、３＝街区、４＝号</remarks>
    Private ReadOnly AddressCodeLengths_() As Integer = {AddressCodeLength.Pref, AddressCodeLength.City, _
                                                         AddressCodeLength.Street, AddressCodeLength.Block, _
                                                         AddressCodeLength.House}
    ''' <summary>
    ''' 住所レベル配列
    ''' </summary>
    ''' <remarks>インデックス番号：０＝都道府県、１＝市区町村、２＝大字・丁目、３＝街区、４＝号</remarks>
    Private ReadOnly AddressLevels_() As Integer = {AddressLevel.Pref, AddressLevel.City, _
                                                    AddressLevel.Street, AddressLevel.Block, _
                                                    AddressLevel.House}

#End Region

    '----------------------------------------------------------------------------------------------
    '	Private Fields
    '----------------------------------------------------------------------------------------------
#Region "Private Fields"

    ''' <summary>
    ''' ユーザオブジェクト
    ''' </summary>
    Private user_ As User

    ''' <summary>
    ''' 検索モード
    ''' </summary>
    ''' <remarks></remarks>
    Private searchMode_ As SearchMode

    ''' <summary>
    ''' 検索結果イテム
    ''' </summary>
    ''' <remarks></remarks>
    Private resultSearchItem_ As GCService.SearchAddressItem

    ''' <summary>
    ''' 絞り込み検索用住所レベル別一覧データリスト
    ''' </summary>
    ''' <remarks>インデックス番号：０＝都道府県、１＝市区町村、２＝大字・丁目、３＝街区、４＝号</remarks>
    Private enumAddressDataList_ As New List(Of EnumAddressData)

#End Region

    '----------------------------------------------------------------------------------------------
    '	Properties
    '----------------------------------------------------------------------------------------------
#Region "Properties"

    ''' <summary>
    ''' ユーザオブジェクトを取得・設定する
    ''' </summary>
    Private Property User() As User
        Get
            If Me.user_ Is Nothing Then
                Throw New Exception("ユーザ認証と計算用道路データの選択が行われていません。")
            End If
            Return Me.user_

        End Get
        Set(ByVal value As User)
            Me.user_ = value
        End Set
    End Property

    ''' <summary>
    ''' 地点名を取得・設定する
    ''' </summary>
    ''' <value></value>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Public Property PointName() As String
        Get
            Return textBoxLocationName.Text
        End Get
        Set(ByVal value As String)
            textBoxLocationName.Text = value
        End Set
    End Property

    ''' <summary>
    ''' 検索結果住所を取得する
    ''' </summary>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Public ReadOnly Property ResultAddress() As String
        Get
            Return textBoxResultString.Text
        End Get
    End Property

    ''' <summary>
    ''' 検索結果経度を取得する
    ''' </summary>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Public ReadOnly Property ResultLon() As String
        Get
            Return textBoxCurrentLon.Text
        End Get
    End Property

    ''' <summary>
    ''' 検索結果緯度を取得する
    ''' </summary>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Public ReadOnly Property ResultLat() As String
        Get
            Return textBoxCurrentLat.Text
        End Get
    End Property

#End Region

    '----------------------------------------------------------------------------------------------
    '	Public Methods
    '----------------------------------------------------------------------------------------------
#Region "Public Methods"

    ''' <summary>
    ''' コンストラクタ
    ''' </summary>
    ''' <param name="user">ユーザオブジェクト</param>
    ''' <param name="searchMode">検索モード</param>
    ''' <remarks></remarks>
    Sub New(ByVal user As User, ByVal searchMode As SearchMode)
        ' この呼び出しは、Windows フォーム デザイナで必要です。
        InitializeComponent()

        ' InitializeComponent() 呼び出しの後で初期化を追加します。
        Me.user_ = user
        Me.searchMode_ = searchMode

        ' 住所レベル別住所データ作成
        Me.enumAddressDataList_.Add(New EnumAddressData(listViewPref, Nothing))                ' 都道府県
        Me.enumAddressDataList_.Add(New EnumAddressData(listViewCity, textBoxFilterCity))      ' 市区町村
        Me.enumAddressDataList_.Add(New EnumAddressData(listViewStreet, textBoxFilterStreet))  ' 大字・丁目
        Me.enumAddressDataList_.Add(New EnumAddressData(listViewBlock, textBoxFilterBlock))    ' 街区
        Me.enumAddressDataList_.Add(New EnumAddressData(listViewHouse, textBoxFilterHouse))    ' 号
    End Sub

#End Region

    '----------------------------------------------------------------------------------------------
    '   Shared Methods
    '----------------------------------------------------------------------------------------------
#Region "Shared Methods"

    ''' <summary>
    ''' 住所名を連結する
    ''' </summary>
    ''' <param name="address1">連結される住所名</param>
    ''' <param name="address2">連結する住所名</param>
    ''' <returns>連結後の住所名</returns>
    ''' <remarks></remarks>
    Private Shared Function ConcatenateAddress(ByVal address1 As String, ByVal address2 As String) As String
        Dim returnValue As String                           ' 戻り値

        Dim regexNashi As New Regex("^（.+なし）$")         ' "（～なし）"用正規表現
        If regexNashi.IsMatch(address2) Then
            '"（～なし）"文字列であれば連結しない
            returnValue = address1
        Else
            ' 数字連続判定
            Dim regexTailNum As New Regex("[0-9０-９]$")    ' 末尾数字用正規表現
            Dim regexHeadNum As New Regex("^[0-9０-９]")    ' 先頭数字用正規表現
            If regexTailNum.IsMatch(address1) AndAlso regexHeadNum.IsMatch(address2) Then _
                ' 数字が連続する場合、ハイフンを挟む
                returnValue = String.Format("{0}‐{1}", address1, address2)
            Else
                returnValue = address1 + address2
            End If

        End If

        Return returnValue
    End Function

    ''' <summary>
    ''' 指定した住所レベルの長さになるように住所コードの右側を削除する
    ''' </summary>
    ''' <param name="addressCode">元となる住所コード</param>
    ''' <param name="addressLevel">住所レベル</param>
    ''' <returns>指定した住所レベルの住所コード</returns>
    ''' <remarks></remarks>
    Private Shared Function TrimZeroAddressCode(ByVal addressCode As String, ByVal addressLevel As Int32) As String
        Select Case addressLevel
            Case Act.Dcws.AddressLevel.Pref
                TrimZeroAddressCode = addressCode.Substring(0, AddressCodeLength.Pref)
            Case Act.Dcws.AddressLevel.City
                TrimZeroAddressCode = addressCode.Substring(0, AddressCodeLength.City)
            Case Act.Dcws.AddressLevel.Street
                TrimZeroAddressCode = addressCode.Substring(0, AddressCodeLength.Street)
            Case Act.Dcws.AddressLevel.Block
                TrimZeroAddressCode = addressCode.Substring(0, AddressCodeLength.Block)
            Case Act.Dcws.AddressLevel.House
                TrimZeroAddressCode = addressCode.Substring(0, AddressCodeLength.House)
            Case Else
                MessageBox.Show("TrimZeroAddressCodeで異常な住所レベル(" & addressLevel & ")が指定されました")
                TrimZeroAddressCode = addressCode
        End Select
    End Function

    ''' <summary>
    ''' ユーザが郵便番号検索を利用できるかを調べる
    ''' </summary>
    ''' <param name="addressLevel">ユーザの最大住所検索可能レベル</param>
    ''' <returns>郵便番号検索が利用できる場合はTrue。利用できない場合はFalse。</returns>
    ''' <remarks></remarks>
    Private Shared Function CanSearchZipCode(ByVal addressLevel As AddressLevel) As Boolean
        CanSearchZipCode = False
        Select Case addressLevel
            Case Act.Dcws.AddressLevel.None
                '"なし"
            Case Act.Dcws.AddressLevel.Pref
                '"都道府県"
            Case Act.Dcws.AddressLevel.City
                '"市区町村"
            Case Act.Dcws.AddressLevel.Street
                '"大字・丁目"
                CanSearchZipCode = True
            Case Act.Dcws.AddressLevel.Block
                '"街区"
                CanSearchZipCode = True
            Case Act.Dcws.AddressLevel.House
                '"号"
                CanSearchZipCode = True
        End Select
    End Function

#End Region

    '----------------------------------------------------------------------------------------------
    '	Private Methods
    '----------------------------------------------------------------------------------------------
#Region "Private Methods"

    ''' <summary>
    ''' 住所検索サービスオブジェクトと環境設定オブジェクトを作成する
    ''' </summary>
    ''' <param name="gcService">住所検索サービスオブジェクト</param>
    ''' <param name="gcEnviron">環境設定オブジェクト</param>
    ''' <returns>成功時はtrue。失敗時はfalse。</returns>
    Private Function CreateGCServiceObject(ByRef gcService As GCService.ACTGCWS, ByRef gcEnviron As GCService.Environ) As Boolean
        '-------------------------------------------------------------------------------------
        '	管理サービスから住所検索サーバー情報を取得する
        '-------------------------------------------------------------------------------------
        ' 管理サービスのインスタンス作成
        Dim adminService As DCAdmin.ACTDCADMIN = New DCAdmin.ACTDCADMIN()   ' 管理サービス
        adminService.Url = Me.User.AdminUrl                                 ' 管理サービスURL設定
        Dim adminEnviron As DCAdmin.Environ = Me.User.GetAdminEnviron()     ' 管理サービス用環境設定

        ' 住所検索サーバーを取得
        Try
            If Not adminService.GetUsableGeoServer(adminEnviron) Then
                ' 住所検索サーバ情報の取得に失敗
                MessageBox.Show(Me, "住所検索サーバ情報の取得に失敗しました。", "管理サーバ", MessageBoxButtons.OK, MessageBoxIcon.Error)
                gcService = Nothing
                gcEnviron = Nothing
                Return False
            End If
        Catch ex As Exception
            MessageBox.Show(Me, ex.Message, "管理サーバ", MessageBoxButtons.OK, MessageBoxIcon.Error)
            gcService = Nothing
            gcEnviron = Nothing
            Return False
        End Try

        '-------------------------------------------------------------------------------------
        '	住所検索サービスの環境設定をする
        '-------------------------------------------------------------------------------------
        ' 住所検索サービスのインスタンス作成
        gcService = New GCService.ACTGCWS()
        gcService.Url = Utilities.ChangeProtocol(adminEnviron.WSURL, Me.User.SslMode)    ' 住所検索サーバURL
        gcEnviron = Utilities.GetGCServiceEnviron(adminEnviron)                            ' 住所検索サービス環境設定

        Return True
    End Function

    ''' <summary>
    ''' 現在の検索結果情報を再描画
    ''' </summary>
    ''' <remarks></remarks>
    Private Sub RedrawCurrentReusult()
        ' クリア
        textBoxResultString.Text = ""
        textBoxCurrentAddressCode.Text = ""
        textBoxCurrentLon.Text = ""
        textBoxCurrentLat.Text = ""

        ' 結果住所
        Dim resultString As String = ""                     ' 結果住所に表示する文字列
        For Each item As EnumAddressData In Me.enumAddressDataList_
            If Not IsNothing(item.CurrentItem) Then
                resultString = ConcatenateAddress(resultString, item.CurrentItem.Address)
            End If
        Next
        textBoxResultString.Text = resultString

        ' 住所コードと経緯度
        textBoxCurrentAddressCode.Text = resultSearchItem_.AddressCode
        textBoxCurrentLon.Text = resultSearchItem_.Lon
        textBoxCurrentLat.Text = resultSearchItem_.Lat

    End Sub

    ''' <summary>
    ''' 下位住所検索実行
    ''' </summary>
    ''' <param name="addressCode">検索する住所コード</param>
    ''' <param name="enumAddressData">変更する絞り込み検索用住所レベル別一覧データ</param>
    ''' <returns>検索成功時はTrue。検索失敗時はFalse。</returns>
    ''' <remarks></remarks>
    Private Function EnumLowerAddressItemsFromCode(ByVal addressCode As String, ByRef enumAddressData As EnumAddressData) As Boolean
        Dim success As Boolean = False

        Try
            Me.Cursor = Cursors.WaitCursor

            Dim addressLv As Integer = 0
            Select Case addressCode.Length
                Case 0
                    addressLv = AddressLevel.Pref       ' 住所レベルがない場合、都道府県レベル検索
                Case AddressCodeLength.Pref
                    addressLv = AddressLevel.City       ' 住所レベルが都道府県の場合、市区町村レベル検索
                Case AddressCodeLength.City
                    addressLv = AddressLevel.Street     ' 住所レベルが市区町村の場合、大字・丁目レベル検索
                Case AddressCodeLength.Street
                    addressLv = AddressLevel.Block      ' 住所レベルが大字・丁目の場合、街区レベル検索
                Case AddressCodeLength.Block
                    addressLv = AddressLevel.House      ' 住所レベルが街区の場合、号レベル検索
                Case AddressCodeLength.House
                    Return success                      ' 住所レベルが号の場合、失敗
                Case Else
                    MessageBox.Show(Me, String.Format("EnumLowerAddressItemsFromCodeで例外発生:{0}AddressCode({1})の文字長({2})が異常です", _
                                                  vbCrLf, addressCode, addressCode.Length))
                    Return success
            End Select
            If addressLv > Me.User.UserInfo.GeoMaxAddressLevel Then
                ' 最大住所検索レベルより検索レベルが大きい場合は終了
                success = True
                Return success
            End If

            ' 住所検索サービスオブジェクト作成
            Dim gcService As New GCService.ACTGCWS
            Dim gcEnviron As New GCService.Environ
            If Not CreateGCServiceObject(gcService, gcEnviron) Then
                Return success
            End If

            ' パラメータ設定
            Dim param As GCService.CommonParam = New GCService.CommonParam
            param.Datum = Act.Dcws.Datum.Tokyo

            ' 結果受取り用変数
            Dim items As GCService.AddressItem() = Nothing
            Dim geoInfo As GCService.GeoInfo = Nothing

            ' サービス呼出
            WaitForm.WaitCalcTime(Me.User, CalcWaitType.GeoEnum, False, Me)
            Dim resultCode As Integer = gcService.EnumLowerAddressByCode(gcEnviron, param, addressCode, items, geoInfo)
            Me.User.UpdateLastGCTime()
            If resultCode = 0 Then
                enumAddressData.CopyToShowItems(items)
                success = True
            Else
                ' 検索失敗
                MessageBox.Show(Me, "検索サービスの呼び出しに失敗しました。", "住所検索失敗", MessageBoxButtons.OK, MessageBoxIcon.Error)
                Return success
            End If
        Catch ex As Exception
            MessageBox.Show(Me, "EnumLowerAddressItemsFromCodeで例外発生:" & vbCrLf & ex.Message & vbCrLf & ex.StackTrace)
            success = False
        Finally
            Me.Cursor = Cursors.Default
        End Try

        Return success
    End Function

    ''' <summary>
    ''' 検索実行
    ''' </summary>
    ''' <returns>検索成功時はTrue。検索失敗時はFalse。</returns>
    ''' <remarks></remarks>
    Private Function Search() As Boolean
        Dim success As Boolean = False

        If Me.searchMode_ = SearchMode.Address Then
            success = SearchAddress()
        ElseIf (Me.searchMode_ = SearchMode.Zip) Then
            If radioButtonZipSelectCenter.Checked Then
                success = SearchZip()
            Else
                success = EnumAddressByZip()
            End If
        End If

        Return success
    End Function

    ''' <summary>
    ''' 住所検索実行
    ''' </summary>
    ''' <returns>検索成功時はTrue。検索失敗時はFalse。</returns>
    ''' <remarks></remarks>
    Private Function SearchAddress() As Boolean
        Dim success As Boolean = False

        Try
            Me.Cursor = Cursors.WaitCursor

            ' クリア
            textBoxResultString.Text = ""
            textBoxCurrentAddressCode.Text = ""
            textBoxCurrentLon.Text = ""
            textBoxCurrentLat.Text = ""

            ' 初期化
            resultSearchItem_ = Nothing
            For i As Integer = 0 To Me.enumAddressDataList_.Count - 1
                Me.enumAddressDataList_(i).ClearAll()
            Next

            ' 住所検索サービスオブジェクト作成
            Dim gcService As New GCService.ACTGCWS
            Dim gcEnviron As New GCService.Environ
            If Not CreateGCServiceObject(gcService, gcEnviron) Then
                Return success
            End If

            ' パラメータ設定
            Dim param As GCService.CommonParam = New GCService.CommonParam
            param.Datum = Act.Dcws.Datum.Tokyo
            param.SearchFlag = 0

            ' 住所検索アイテム設定
            Dim items(0) As GCService.SearchAddressItem
            items(0) = New GCService.SearchAddressItem
            items(0).SearchAddress = textBoxSearchString.Text

            ' 受取り用住所データ情報
            Dim geoInfo As GCService.GeoInfo = Nothing

            ' サービス呼出
            WaitForm.WaitCalcTime(Me.User, CalcWaitType.GeoSearch, False, Me)
            Dim resultCode As Integer = gcService.SearchAddress(gcEnviron, param, Me.User.UserInfo.GeoMaxAddressLevel, items, geoInfo)
            Me.User.UpdateLastGCTime()
            If resultCode = 0 Then
                Me.resultSearchItem_ = items(0)

                textBoxResultString.Text = Me.resultSearchItem_.AnalyzedAddress
                textBoxCurrentAddressCode.Text = Me.resultSearchItem_.AddressCode
                textBoxCurrentLon.Text = Me.resultSearchItem_.Lon
                textBoxCurrentLat.Text = Me.resultSearchItem_.Lat
            Else
                MessageBox.Show(Me, "住所検索サービスの呼び出しに失敗しました。", "住所検索失敗", MessageBoxButtons.OK, MessageBoxIcon.Error)
                Return success
            End If

            success = True
        Catch ex As Exception
            MessageBox.Show(Me, "SearchAddressで例外発生:" & vbCrLf & ex.Message & vbCrLf & ex.StackTrace, _
                            "エラー", MessageBoxButtons.OK, MessageBoxIcon.Error)
        Finally
            Me.Cursor = Cursors.Default
        End Try

        Return success
    End Function

    ''' <summary>
    ''' 郵便番号検索実行
    ''' </summary>
    ''' <returns>検索成功時はTrue。検索失敗時はFalse。</returns>
    ''' <remarks></remarks>
    Private Function SearchZip() As Boolean
        Dim success As Boolean = False

        ''郵便番号検索の利用制限チェック
        If (Not CanSearchZipCode(Me.User.UserInfo.GeoMaxAddressLevel)) Then
            MessageBox.Show(Me, "現在の会員種別では、郵便番号検索を利用できません。", "利用制限", MessageBoxButtons.OK, MessageBoxIcon.Exclamation)
            Return success
        End If

        Try
            Me.Cursor = Cursors.WaitCursor

            ' クリア
            textBoxResultString.Text = ""
            textBoxCurrentAddressCode.Text = ""
            textBoxCurrentLon.Text = ""
            textBoxCurrentLat.Text = ""

            ' 初期化
            resultSearchItem_ = Nothing
            For i As Integer = 0 To Me.enumAddressDataList_.Count - 1
                Me.enumAddressDataList_(i).ClearAll()
            Next

            ' 住所検索サービスオブジェクト作成
            Dim gcService As New GCService.ACTGCWS
            Dim gcEnviron As New GCService.Environ
            If Not CreateGCServiceObject(gcService, gcEnviron) Then
                Return success
            End If

            ' パラメータ設定
            Dim param As GCService.CommonParam = New GCService.CommonParam()
            param.Datum = Act.Dcws.Datum.Tokyo

            ' 郵便番号検索アイテム設定
            Dim items As GCService.SearchZipItem() = New GCService.SearchZipItem(0) {}
            items(0) = New GCService.SearchZipItem()
            items(0).SearchZip = textBoxSearchString.Text

            ' 結果受取り用変数
            Dim geoInfo As GCService.GeoInfo = Nothing

            ' サービス呼出
            WaitForm.WaitCalcTime(Me.User, CalcWaitType.GeoSearch, False, Me)
            Dim resultCode As Integer = gcService.SearchZip(gcEnviron, param, items, geoInfo)
            Me.User.UpdateLastGCTime()
            If resultCode = 0 Then
                Me.resultSearchItem_ = New GCService.SearchAddressItem()
                Me.resultSearchItem_.SearchAddress = textBoxSearchString.Text
                Me.resultSearchItem_.AddressLevel = items(0).AddressLevel
                Me.resultSearchItem_.AnalyzedAddress = items(0).FullAddress
                Me.resultSearchItem_.AddressCode = items(0).AddressCode
                Me.resultSearchItem_.Lon = items(0).Lon
                Me.resultSearchItem_.Lat = items(0).Lat

                textBoxResultString.Text = Me.resultSearchItem_.AnalyzedAddress
                textBoxCurrentAddressCode.Text = Me.resultSearchItem_.AddressCode
                textBoxCurrentLon.Text = Me.resultSearchItem_.Lon
                textBoxCurrentLat.Text = Me.resultSearchItem_.Lat
            Else
                MessageBox.Show(Me, "住所検索サービスの呼び出しに失敗しました。", "住所検索失敗", MessageBoxButtons.OK, MessageBoxIcon.Error)
                Return success
            End If

            success = True
        Catch ex As Exception
            MessageBox.Show(Me, "SearchZipで例外発生:" & vbCrLf & ex.Message & vbCrLf & ex.StackTrace, _
                            "エラー", MessageBoxButtons.OK, MessageBoxIcon.Error)
        Finally
            Me.Cursor = Cursors.Default
        End Try

        Return success
    End Function

    ''' <summary>
    ''' 郵便番号対応住所列挙実行
    ''' </summary>
    ''' <returns>検索成功時はTrue。検索失敗時はFalse。</returns>
    ''' <remarks></remarks>
    Private Function EnumAddressByZip() As Boolean
        Dim success As Boolean = False

        ''郵便番号検索の利用制限チェック
        If (Not CanSearchZipCode(Me.User.UserInfo.GeoMaxAddressLevel)) Then
            MessageBox.Show(Me, "現在の会員種別では、郵便番号検索を利用できません。", "利用制限", MessageBoxButtons.OK, MessageBoxIcon.Exclamation)
            Return success
        End If

        Try
            Me.Cursor = Cursors.WaitCursor

            ' クリア
            textBoxResultString.Text = ""
            textBoxCurrentAddressCode.Text = ""
            textBoxCurrentLon.Text = ""
            textBoxCurrentLat.Text = ""

            ' 初期化
            resultSearchItem_ = Nothing
            For i As Integer = 0 To Me.enumAddressDataList_.Count - 1
                Me.enumAddressDataList_(i).ClearAll()
            Next

            ' 住所検索サービスオブジェクト作成
            Dim gcService As New GCService.ACTGCWS
            Dim gcEnviron As New GCService.Environ
            If Not CreateGCServiceObject(gcService, gcEnviron) Then
                Return success
            End If

            ' パラメータ設定
            Dim param As GCService.CommonParam = New GCService.CommonParam()
            param.Datum = Act.Dcws.Datum.Tokyo

            ' 結果受取り用変数
            Dim zipItems As GCService.ZipItem() = Nothing
            Dim geoInfo As GCService.GeoInfo = Nothing

            ' サービス呼出
            WaitForm.WaitCalcTime(Me.User, CalcWaitType.GeoEnum, False, Me)
            Dim resultCode As Integer = gcService.EnumAddressByZip(gcEnviron, param, textBoxSearchString.Text, zipItems, geoInfo)
            Me.User.UpdateLastGCTime()
            If resultCode = 0 Then
                Dim selectItem As GCService.ZipItem = Nothing
                If zipItems.Length = 1 Then
                    ' 結果がひとつの場合
                    selectItem = zipItems(0)
                ElseIf zipItems.Length >= 2 Then
                    ' 結果が２以上の場合
                    Using selectForm As New SelectAddressForm(zipItems)
                        If selectForm.ShowDialog(Me) = Windows.Forms.DialogResult.OK Then
                            selectItem = selectForm.SelectedItem
                        End If
                    End Using
                End If

                If selectItem Is Nothing Then
                    Me.resultSearchItem_ = New GCService.SearchAddressItem
                    Me.resultSearchItem_.SearchAddress = textBoxSearchString.Text
                    Me.resultSearchItem_.AnalyzedAddress = ""
                    Me.resultSearchItem_.AddressCode = ""
                    Me.resultSearchItem_.Lon = ""
                    Me.resultSearchItem_.Lat = ""
                Else
                    Me.resultSearchItem_ = New GCService.SearchAddressItem()
                    Me.resultSearchItem_.SearchAddress = textBoxSearchString.Text
                    Me.resultSearchItem_.AnalyzedAddress = selectItem.FullAddress
                    Me.resultSearchItem_.AddressCode = selectItem.AddressCode
                    Me.resultSearchItem_.Lon = selectItem.Lon
                    Me.resultSearchItem_.Lat = selectItem.Lat
                End If

                textBoxResultString.Text = Me.resultSearchItem_.AnalyzedAddress
                textBoxCurrentAddressCode.Text = Me.resultSearchItem_.AddressCode
                textBoxCurrentLon.Text = Me.resultSearchItem_.Lon
                textBoxCurrentLat.Text = Me.resultSearchItem_.Lat
            Else
                MessageBox.Show(Me, "住所検索サービスの呼び出しに失敗しました。", "住所検索失敗", MessageBoxButtons.OK, MessageBoxIcon.Error)
                Return success
            End If

            success = True
        Catch ex As Exception
            MessageBox.Show(Me, "EnumAddressByZipで例外発生:" & vbCrLf & ex.Message & vbCrLf & ex.StackTrace, _
                            "エラー", MessageBoxButtons.OK, MessageBoxIcon.Error)
        Finally
            Me.Cursor = Cursors.Default
        End Try

        Return success
    End Function

    ''' <summary>
    ''' リストビューを使用可能にする
    ''' </summary>
    ''' <remarks></remarks>
    Private Sub EnableListView()
        ' リストビュー
        listViewPref.Enabled = True
        listViewCity.Enabled = True
        listViewStreet.Enabled = True
        listViewBlock.Enabled = True
        listViewHouse.Enabled = True
        labelPref.Enabled = True
        labelCity.Enabled = True
        labelStreet.Enabled = True
        labelBlock.Enabled = True
        labelHouse.Enabled = True

        ' フィルター
        labelFilter.Enabled = True
        buttonClearAllFilter.Enabled = True
        textBoxFilterCity.Enabled = True
        textBoxFilterStreet.Enabled = True
        textBoxFilterBlock.Enabled = True
        textBoxFilterHouse.Enabled = True
    End Sub

    ''' <summary>
    ''' リストビューを使用不可にする
    ''' </summary>
    ''' <remarks></remarks>
    Private Sub DisableListView()
        ' リストビュー
        listViewPref.Items.Clear()
        listViewCity.Items.Clear()
        listViewStreet.Items.Clear()
        listViewBlock.Items.Clear()
        listViewHouse.Items.Clear()
        listViewPref.AutoResizeColumn(0, ColumnHeaderAutoResizeStyle.ColumnContent)
        listViewCity.AutoResizeColumn(0, ColumnHeaderAutoResizeStyle.ColumnContent)
        listViewStreet.AutoResizeColumn(0, ColumnHeaderAutoResizeStyle.ColumnContent)
        listViewBlock.AutoResizeColumn(0, ColumnHeaderAutoResizeStyle.ColumnContent)
        listViewHouse.AutoResizeColumn(0, ColumnHeaderAutoResizeStyle.ColumnContent)
        listViewPref.Enabled = False
        listViewCity.Enabled = False
        listViewStreet.Enabled = False
        listViewBlock.Enabled = False
        listViewHouse.Enabled = False
        labelPref.Enabled = False
        labelCity.Enabled = False
        labelStreet.Enabled = False
        labelBlock.Enabled = False
        labelHouse.Enabled = False

        ' フィルター
        labelFilter.Enabled = False
        buttonClearAllFilter.Enabled = False
        textBoxFilterCity.Enabled = False
        textBoxFilterStreet.Enabled = False
        textBoxFilterBlock.Enabled = False
        textBoxFilterHouse.Enabled = False
        textBoxFilterCity.Text = ""
        textBoxFilterStreet.Text = ""
        textBoxFilterBlock.Text = ""
        textBoxFilterHouse.Text = ""
    End Sub

#End Region

    '----------------------------------------------------------------------------------------------
    '	Event Handlers
    '----------------------------------------------------------------------------------------------
#Region "Event Handlers"

    ''' <summary>
    ''' フォームロード時
    ''' </summary>
    ''' <param name="sender"></param>
    ''' <param name="e"></param>
    ''' <remarks></remarks>
    Private Sub SearchAddressForm_Load(ByVal sender As Object, ByVal e As System.EventArgs) Handles MyBase.Load
        ' 検索モードによってテキスト変更
        If Me.searchMode_ = SearchMode.Zip Then
            Me.labelSearchString.Text = "検索対象７桁郵便番号"
            Me.Text = "郵便番号検索"
            Me.labelZipSelectType.Visible = True
            Me.radioButtonZipSelectCenter.Visible = True
            Me.radioButtonZipSelectList.Visible = True
            Me.textBoxSearchString.Size = New System.Drawing.Size(172, 32)
        Else
            Me.labelSearchString.Text = "検索対象漢字住所"
            Me.Text = "住所検索"
            Me.labelZipSelectType.Visible = False
            Me.radioButtonZipSelectCenter.Visible = False
            Me.radioButtonZipSelectList.Visible = False
            Me.textBoxSearchString.Size = New System.Drawing.Size(488, 32)
        End If

        'リストビューを使用不可にする
        DisableListView()

    End Sub

    ''' <summary>
    ''' 検索開始ボタン
    ''' </summary>
    ''' <param name="sender"></param>
    ''' <param name="e"></param>
    ''' <remarks></remarks>
    Private Sub buttonSearch_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles buttonSearch.Click
        Try
            ' 検索実行
            If Search() Then
                '絞り込み検索にチェックが入っていれば、表示を変更する
                If checkBoxSelect.Checked Then
                    checkBoxSelect_CheckedChanged(New Object, New System.EventArgs)
                End If
            Else
                '検索失敗時
                checkBoxSelect.Checked = False
            End If

        Catch ex As Exception
            MessageBox.Show(Me, "buttonSearch_Clickで例外発生:" & vbCrLf & ex.Message & vbCrLf & ex.StackTrace)
        End Try

    End Sub

    ''' <summary>
    ''' 絞り込み検索チェックボックスのチェック変更時
    ''' </summary>
    ''' <param name="sender"></param>
    ''' <param name="e"></param>
    ''' <remarks></remarks>
    Private Sub checkBoxSelect_CheckedChanged(ByVal sender As Object, ByVal e As System.EventArgs) Handles checkBoxSelect.CheckedChanged
        Try
            If checkBoxSelect.Checked Then
                'リストビューを使用可能にする
                EnableListView()

                ' クリア
                For i As Integer = 0 To Me.enumAddressDataList_.Count - 1
                    Me.enumAddressDataList_(i).ClearList()
                    Me.enumAddressDataList_(i).ClearCurrent()
                Next

                ' 住所レベル別住所アイテム取得
                If Me.resultSearchItem_ Is Nothing OrElse Me.resultSearchItem_.SearchAddress <> textBoxSearchString.Text Then
                    ' 検索結果がない、または前回の検索文字列と検索文字列が異なる場合
                    If Not Search() Then
                        ' 検索失敗
                        checkBoxSelect.Checked = False
                        Exit Sub
                    End If
                End If

                '' 住所レベル別住所コード作成
                Dim addressCodes As List(Of String) = New List(Of String)()
                For Each length As Integer In AddressCodeLengths_
                    If Me.resultSearchItem_.AddressCode.Length >= length Then
                        addressCodes.Add(Me.resultSearchItem_.AddressCode.Substring(0, length))
                    Else
                        Exit For
                    End If
                Next

                ' 各住所レベルの住所を取得
                Dim resultEnum As Boolean = True        ' 下位住所列挙の結果
                For i As Integer = 0 To Me.enumAddressDataList_.Count - 1
                    If i = 0 Then
                        ' 都道府県
                        resultEnum = EnumLowerAddressItemsFromCode("", Me.enumAddressDataList_(i))
                    ElseIf Not IsNothing(Me.enumAddressDataList_(i - 1).EnumItems) AndAlso (addressCodes.Count > i - 1) Then
                        ' 市区町村から号
                        resultEnum = EnumLowerAddressItemsFromCode(addressCodes(i - 1), Me.enumAddressDataList_(i))
                    End If
                    If Not resultEnum Then
                        ' 取得失敗
                        checkBoxSelect.Checked = False
                        Exit Sub
                    End If
                Next
                For i As Integer = 0 To addressCodes.Count - 1
                    Me.enumAddressDataList_(i).SelectCurrentItem(addressCodes(i))
                Next

                ' 各レベルをリストビューに再描画
                For i As Integer = 0 To Me.enumAddressDataList_.Count - 1
                    Me.enumAddressDataList_(i).RedrawListView(-1)
                Next
            Else
                'リストビューを使用不能にする
                DisableListView()
            End If
        Catch ex As Exception
            MessageBox.Show(Me, "checkBoxSelect_CheckedChangedで例外発生:" & vbCrLf & ex.Message & vbCrLf & ex.StackTrace)
            checkBoxSelect.Checked = False
        End Try

    End Sub

    ''' <summary>
    ''' リストビュークリック時
    ''' </summary>
    ''' <param name="sender"></param>
    ''' <param name="e"></param>
    ''' <remarks></remarks>
    Private Sub listView_Click(ByVal sender As Object, ByVal e As System.EventArgs) Handles listViewPref.Click, listViewCity.Click, listViewStreet.Click, listViewBlock.Click, listViewHouse.Click

        Try
            Dim listView As ListView = DirectCast(sender, ListView)             ' クリックされたリストビュー
            If listView.SelectedItems.Count <> 1 Then
                ' 選択アイテムが１つ以外の場合は中止
                Exit Sub
            End If

            Dim selectedListItem As ListViewItem = listView.SelectedItems(0)    ' 選択されているリストアイテム
            Dim topItemIndex As Int32 = listView.TopItem.Index                  ' リストビューの一番上に表示されているアイテムインデックス

            ' 選択された住所レベルインデックス取得
            Dim currentLevelIndex As Integer = 0
            For i As Integer = 0 To Me.enumAddressDataList_.Count - 1
                If Me.enumAddressDataList_(i).ListView.Name = listView.Name Then
                    currentLevelIndex = i
                    Exit For
                End If
            Next

            ' 下位のレベルをクリア
            For i As Integer = currentLevelIndex + 1 To Me.enumAddressDataList_.Count - 1
                Me.enumAddressDataList_(i).ClearAll()
            Next

            ' リストビューで選択したアイテムを設定            
            Me.enumAddressDataList_(currentLevelIndex).SelectCurrentItem(selectedListItem.Index)
            resultSearchItem_ = New SampleMG.GCService.SearchAddressItem()
            resultSearchItem_.AddressCode = Me.enumAddressDataList_(currentLevelIndex).CurrentItem.AddressCode
            resultSearchItem_.AddressLevel = Me.enumAddressDataList_(currentLevelIndex).CurrentItem.AddressLevel
            resultSearchItem_.AnalyzedAddress = Me.enumAddressDataList_(currentLevelIndex).CurrentItem.Address
            resultSearchItem_.Lon = Me.enumAddressDataList_(currentLevelIndex).CurrentItem.Lon
            resultSearchItem_.Lat = Me.enumAddressDataList_(currentLevelIndex).CurrentItem.Lat

            ' 下位の住所レベルリストを取得
            If Me.enumAddressDataList_(currentLevelIndex).CurrentItem.LowerAddressLevel <> AddressLevel.None AndAlso _
                currentLevelIndex < Me.enumAddressDataList_.Count - 1 AndAlso _
                Not IsNothing(Me.enumAddressDataList_(currentLevelIndex).CurrentItem) Then
                If Not EnumLowerAddressItemsFromCode(Me.enumAddressDataList_(currentLevelIndex).CurrentItem.AddressCode, _
                                                     Me.enumAddressDataList_(currentLevelIndex + 1)) Then
                    '　下位住所取得失敗
                    checkBoxSelect.Checked = False
                    Exit Sub
                End If
            End If

            ' カレント住所をラベル表示
            RedrawCurrentReusult()

            ' リストを再描画
            For i As Integer = 0 To Me.enumAddressDataList_.Count - 1 - currentLevelIndex
                If currentLevelIndex = Me.enumAddressDataList_.Count - 1 - i Then
                    ' 選択された住所レベルの場合、トップアイテム設定
                    Me.enumAddressDataList_(Me.enumAddressDataList_.Count - 1 - i).RedrawListView(topItemIndex)
                Else
                    Me.enumAddressDataList_(Me.enumAddressDataList_.Count - 1 - i).RedrawListView(-1)
                End If
            Next
            Me.enumAddressDataList_(currentLevelIndex).ListView.Focus()

        Catch ex As Exception
            MessageBox.Show(Me, "listvPref_Clickで例外発生:" & vbCrLf & ex.Message & vbCrLf & ex.StackTrace)
        End Try
    End Sub

    ''' <summary>
    ''' アドレスコードテキストボックスの内容変更時
    ''' </summary>
    ''' <param name="sender"></param>
    ''' <param name="e"></param>
    ''' <remarks></remarks>
    Private Sub editCurrentAddressCode_TextChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles textBoxCurrentAddressCode.TextChanged
        '「OKボタン」の表示・非表示
        If Me.textBoxCurrentAddressCode.Text.Trim.Length > 0 Then
            buttonOK.Enabled = True
        Else
            buttonOK.Enabled = False
        End If
    End Sub

    ''' <summary>
    ''' 市区町村フィルタテキスト変更
    ''' </summary>
    ''' <param name="sender"></param>
    ''' <param name="e"></param>
    ''' <remarks></remarks>
    Private Sub textBoxFilterCity_TextChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles textBoxFilterCity.TextChanged
        Me.enumAddressDataList_(1).UpdateListViewFilter()
    End Sub

    ''' <summary>
    ''' 大字・丁目フィルタテキスト変更
    ''' </summary>
    ''' <param name="sender"></param>
    ''' <param name="e"></param>
    ''' <remarks></remarks>
    Private Sub textBoxFilterStreet_TextChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles textBoxFilterStreet.TextChanged
        Me.enumAddressDataList_(2).UpdateListViewFilter()
    End Sub

    ''' <summary>
    ''' 街区フィルタテキスト変更
    ''' </summary>
    ''' <param name="sender"></param>
    ''' <param name="e"></param>
    ''' <remarks></remarks>
    Private Sub textBoxFilterBlock_TextChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles textBoxFilterBlock.TextChanged
        Me.enumAddressDataList_(3).UpdateListViewFilter()
    End Sub

    ''' <summary>
    ''' 号フィルタテキスト変更
    ''' </summary>
    ''' <param name="sender"></param>
    ''' <param name="e"></param>
    ''' <remarks></remarks>
    Private Sub textBoxFilterHouse_TextChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles textBoxFilterHouse.TextChanged
        Me.enumAddressDataList_(4).UpdateListViewFilter()
    End Sub

    ''' <summary>
    ''' 全てのフィルターを解除する
    ''' </summary>
    ''' <param name="sender"></param>
    ''' <param name="e"></param>
    ''' <remarks></remarks>
    Private Sub buttonClearAllFilter_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles buttonClearAllFilter.Click
        textBoxFilterCity.Text = ""
        textBoxFilterStreet.Text = ""
        textBoxFilterBlock.Text = ""
        textBoxFilterHouse.Text = ""
    End Sub

#End Region

End Class