﻿'//////////////////////////////////////////////////////////////////////////////////////////////////
' 
'  ACT距離計算サービス サンプルプログラム
'  ユーザクラス
'
'  Copyrights (c) 2011 Advanced Core Technologies, Inc. All rights reserved.
'
'  2011/08/25 Rev.2.2.0.0 ISHI 作成
'
'//////////////////////////////////////////////////////////////////////////////////////////////////

''' <summary>
''' ユーザクラス
''' </summary>
Public Class User
    '----------------------------------------------------------------------------------------------
    '  Public Methods
    '----------------------------------------------------------------------------------------------
#Region "Public Methods"

    ''' <summary>
    ''' コンストラクタ
    ''' </summary>
    ''' <param name="userID">ユーザID</param>
    ''' <param name="password">パスワード</param>
    ''' <param name="adminUrl">管理サービスURL</param>
    ''' <param name="sslMode">SSLモード</param>
    Public Sub New(ByVal userID As String, ByVal password As String, ByVal adminUrl As String, ByVal sslMode As Boolean)
        Me.userID_ = userID
        Me.password_ = password
        Me.adminUrl_ = adminUrl
        Me.sslMode_ = sslMode
    End Sub

    ''' <summary>
    ''' 管理サービス用環境設定オブジェクトを取得する
    ''' </summary>
    ''' <returns>管理サービス用環境設定オブジェクト</returns>
    Public Function GetAdminEnviron() As DCAdmin.Environ
        Dim adminEnviron As DCAdmin.Environ = New DCAdmin.Environ()
        adminEnviron.UserID = Me.userID_
        adminEnviron.UUID = Me.password_
        Return adminEnviron
    End Function

    ''' <summary>
    ''' 最終距離計算日時を更新する
    ''' </summary>
    Public Sub UpdateLastDCTime()
        Me.lastDCTime_ = DateTime.Now
    End Sub

    ''' <summary>
    ''' 最終住所検索日時を更新する
    ''' </summary>
    Public Sub UpdateLastGCTime()
        Me.lastGCTime_ = DateTime.Now
    End Sub

#End Region

    '----------------------------------------------------------------------------------------------
    '  Public Properties
    '----------------------------------------------------------------------------------------------
#Region "Public Properties"

    ''' <summary>
    ''' ユーザIDを取得する
    ''' </summary>
    Public ReadOnly Property UserID() As String
        Get
            Return Me.userID_
        End Get
    End Property

    ''' <summary>
    ''' パスワードを取得する
    ''' </summary>
    Public ReadOnly Property Password() As String
        Get
            Return Me.password_
        End Get
    End Property

    ''' <summary>
    ''' 管理サービスURLを取得する
    ''' </summary>
    Public ReadOnly Property AdminUrl() As String
        Get
            Return Utilities.ChangeProtocol(Me.adminUrl_, Me.sslMode_)
        End Get
    End Property

    ''' <summary>
    ''' SSLモードを取得する
    ''' </summary>   
    Public ReadOnly Property SslMode() As Boolean
        Get
            Return Me.sslMode_
        End Get
    End Property

    ''' <summary>
    ''' 最終距離計算時刻を取得する
    ''' </summary>
    Public ReadOnly Property LastDCTime() As DateTime
        Get
            Return Me.lastDCTime_
        End Get
    End Property

    ''' <summary>
    ''' 最終住所検索時刻を取得する
    ''' </summary>
    Public ReadOnly Property LastGCTime() As DateTime
        Get
            Return Me.lastGCTime_
        End Get
    End Property

    ''' <summary>
    ''' 利用する計算用道路データ情報を取得・設定する
    ''' </summary>
    Public Property NWInfo() As DCAdmin.NWInfo
        Get
            Return Me.nwInfo_
        End Get
        Set(ByVal value As DCAdmin.NWInfo)
            Me.nwInfo_ = value
        End Set
    End Property

    ''' <summary>
    ''' サービスユーザ情報を取得・設定する
    ''' </summary>
    Public Property UserInfo() As DCAdmin.UserInformation2
        Get
            Return Me.userInfo_
        End Get
        Set(ByVal value As DCAdmin.UserInformation2)
            Me.userInfo_ = value
        End Set
    End Property

#End Region

    '----------------------------------------------------------------------------------------------
    '  Private Fields
    '----------------------------------------------------------------------------------------------
#Region "Private Fields"

    ''' <summary>
    ''' ユーザID
    ''' </summary>
    Private userID_ As String

    ''' <summary>
    ''' パスワード
    ''' </summary>
    Private password_ As String

    ''' <summary>
    ''' 管理サービスURL
    ''' </summary>
    Private adminUrl_ As String

    ''' <summary>
    ''' SSLモード
    ''' </summary>
    Private sslMode_ As Boolean

    ''' <summary>
    ''' 最終距離計算時刻
    ''' </summary>
    Private lastDCTime_ As DateTime

    ''' <summary>
    ''' 最終住所検索時刻
    ''' </summary>
    Private lastGCTime_ As DateTime

    ''' <summary>
    ''' 利用する計算用道路データ情報
    ''' </summary>
    Private nwInfo_ As DCAdmin.NWInfo

    ''' <summary>
    ''' サービスユーザ情報
    ''' </summary>
    Private userInfo_ As DCAdmin.UserInformation2

#End Region

End Class
