﻿'//////////////////////////////////////////////////////////////////////////////////////////////////
'  
'   ACT距離計算サービス サンプルプログラム
'   ユーザ情報フォームクラス
' 
'   Copyrights (c) 2011 Advanced Core Technologies, Inc. All rights reserved.
' 
'   2011/09/05 Rev.2.2.0.0 ISHI 初版作成
' 
'//////////////////////////////////////////////////////////////////////////////////////////////////

''' <summary>
''' ユーザ情報フォームクラス
''' </summary>
''' <remarks></remarks>
Public Class UserInfoForm
    '----------------------------------------------------------------------
    '  public methods
    '----------------------------------------------------------------------
#Region "public methods"

    ''' <summary>
    ''' コンストラクタ
    ''' </summary>
    ''' <param name="user">ユーザオブジェクト</param>
    Sub New(ByVal user As User)
        InitializeComponent()

        If User Is Nothing OrElse User.UserInfo Is Nothing Then
            Throw New ArgumentNullException()
        End If
        Me.user_ = user
    End Sub

#End Region

    ' ---------------------------------------------------------------------
    '  private event handlers
    ' ---------------------------------------------------------------------
#Region "private event handlers"

    ''' <summary>
    ''' フォームロード時
    ''' </summary>
    ''' <param name="sender"></param>
    ''' <param name="e"></param>
    ''' <remarks></remarks>
    Private Sub UserInfoForm_Load(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MyBase.Load
        LabelUserNameInfo.Text = Me.user_.UserID                 ' ユーザ名
        LabelAdminServerUrlInfo.Text = Me.user_.AdminUrl         ' 管理サーバURL

        LabelMemberKindInfo.Text = Me.user_.UserInfo.MemberKind  ' 会員種別
        LabelExpiryDateInfo.Text = Me.user_.UserInfo.ExpiryDate  ' 会員期限

        ' 計算間隔（秒）
        If Me.user_.UserInfo.CalcInterval = 0 Then
            LabelCalcIntervalInfo.Text = ContinuationString
        Else
            LabelCalcIntervalInfo.Text = Me.user_.UserInfo.CalcInterval.ToString() + " 秒"
        End If

        ' ルート最大地点数
        If Me.user_.UserInfo.CalcRouteMaxLocs = -1 Then
            LabelCalcRouteMaxLocInfo.Text = UnlimitedString
        Else
            LabelCalcRouteMaxLocInfo.Text = Me.user_.UserInfo.CalcRouteMaxLocs.ToString() + " 地点"
        End If

        ' 最短ルート最大地点数
        If Me.user_.UserInfo.CalcOptRouteMaxLocs = -1 Then
            LabelCalcOptRouteMaxLocInfo.Text = UnlimitedString
        Else
            LabelCalcOptRouteMaxLocInfo.Text = Me.user_.UserInfo.CalcOptRouteMaxLocs.ToString() + " 地点"
        End If

        ' エリア最大時間（0.1秒単位を分で表示する）
        If Me.user_.UserInfo.CalcAreaMaxTime = -1 Then
            LabelCalcAreaMaxTimeInfo.Text = UnlimitedString
        Else
            LabelCalcAreaMaxTimeInfo.Text = (Me.user_.UserInfo.CalcAreaMaxTime / 600).ToString() + " 分"
        End If

        ' エリア最大距離（m単位をkmで表示する）
        If Me.user_.UserInfo.CalcAreaMaxDist = -1 Then
            LabelCalcAreaMaxDistInfo.Text = UnlimitedString
        Else
            LabelCalcAreaMaxDistInfo.Text = (Me.user_.UserInfo.CalcAreaMaxDist / 1000).ToString() + " km"
        End If

        ' 片道一括最大地点数
        If Me.user_.UserInfo.CalcAreaMaxLocs = -1 Then
            LabelCalcOneWayMaxLocInfo.Text = UnlimitedString
        Else
            LabelCalcOneWayMaxLocInfo.Text = Me.user_.UserInfo.CalcAreaMaxLocs.ToString() + " 地点"
        End If

        ' 住所レベル
        Select Case Me.user_.UserInfo.GeoMaxAddressLevel
            Case Act.Dcws.AddressLevel.Pref
                LabelMaxAddressLevelInfo.Text = "都道府県"
            Case Act.Dcws.AddressLevel.City
                LabelMaxAddressLevelInfo.Text = "市区町村"
            Case Act.Dcws.AddressLevel.Street
                LabelMaxAddressLevelInfo.Text = "大字・丁目"
            Case Act.Dcws.AddressLevel.Block
                LabelMaxAddressLevelInfo.Text = "街区"
            Case Act.Dcws.AddressLevel.House
                LabelMaxAddressLevelInfo.Text = "号"
            Case Else
                LabelMaxAddressLevelInfo.Text = "なし"
        End Select

        ' 住所検索間隔（秒）
        If Me.user_.UserInfo.GeoInterval = 0 Then
            LabelSearchAddressIntervalInfo.Text = ContinuationString
        Else
            LabelSearchAddressIntervalInfo.Text = Me.user_.UserInfo.GeoInterval.ToString() + " 秒"
        End If

        ' 下位住所列挙計算間隔（秒）
        If Me.user_.UserInfo.GeoEnumInterval = 0 Then
            LabelEnumAddressIntervalInfo.Text = ContinuationString
        Else
            LabelEnumAddressIntervalInfo.Text = Me.user_.UserInfo.GeoEnumInterval.ToString() + " 秒"
        End If

        ' 住所検索最大地点数
        If Me.user_.UserInfo.GeoMaxLocs = -1 Then
            LabelSearchAddressMaxItemInfo.Text = UnlimitedString
        Else
            LabelSearchAddressMaxItemInfo.Text = Me.user_.UserInfo.GeoMaxLocs.ToString() + " 地点"
        End If

        ' 試用会員用表示
        If Me.user_.UserInfo.MaxCalcCount = -1 Then
            LabelRestCalc.Visible = False
        Else
            LabelRestCalc.Text = "※ 残存計算回数は、" + _
            Math.Max(Me.user_.UserInfo.MaxCalcCount - Me.user_.UserInfo.CalcCount, 0).ToString(+" 回です。")

        End If
    End Sub

#End Region

    ' ---------------------------------------------------------------------
    '  private constants
    ' ---------------------------------------------------------------------
#Region "private constants"

    ''' <summary>
    ''' 無制限文字列
    ''' </summary>
    Private Const UnlimitedString As String = "無制限"

    ''' <summary>
    ''' 連続文字列
    ''' </summary>
    Private Const ContinuationString As String = "連続"

#End Region

    ' ---------------------------------------------------------------------
    '  private fields
    ' ---------------------------------------------------------------------
#Region "private fields"

    ''' <summary>
    ''' ユーザオブジェクト
    ''' </summary>
    Private user_ As User

#End Region

End Class