﻿'//////////////////////////////////////////////////////////////////////////////////////////////////
'  
'   ACT距離計算サービス サンプルプログラム
'   計算待機フォームクラス
' 
'   Copyrights (c) 2011 Advanced Core Technologies, Inc. All rights reserved.
' 
'   2011/11/15 Rev.2.3.0.0 ISHI 公開
' 
'//////////////////////////////////////////////////////////////////////////////////////////////////

''' <summary>
''' 計算待機種別
''' </summary>
Public Enum CalcWaitType
    ''' <summary>
    ''' 距離計算
    ''' </summary>
    DistCalc
    ''' <summary>
    ''' 住所検索
    ''' </summary>
    GeoSearch
    ''' <summary>
    ''' 住所列挙
    ''' </summary>
    GeoEnum
End Enum

''' <summary>
''' 計算待機フォームクラス
''' </summary>
''' <remarks></remarks>
Public Class WaitForm

    '----------------------------------------------------------------------
    '  public static methods
    '----------------------------------------------------------------------
#Region "public static methods"

    ''' <summary>
    ''' 距離計算が可能になるまで待機する
    ''' </summary>
    ''' <param name="user">ユーザオブジェクト</param>
    ''' <param name="waitType">計算待機種別</param>
    ''' <param name="showForm">フォーム表示フラグ</param>
    ''' <param name="owner">オーナーウィンドウ</param>
    ''' <returns>正常終了時True。キャンセル時False。</returns>
    Public Shared Function WaitCalcTime(ByVal user As User, ByVal waitType As CalcWaitType, ByVal showForm As Boolean, ByVal owner As IWin32Window) As Boolean
        Dim lastTime As DateTime = New DateTime()
        Dim calcInterval As Integer = 0
        Select Case waitType
            Case CalcWaitType.DistCalc
                lastTime = user.LastDCTime
                calcInterval = user.UserInfo.CalcInterval
            Case CalcWaitType.GeoSearch
                lastTime = user.LastGCTime
                calcInterval = user.UserInfo.GeoInterval
            Case CalcWaitType.GeoEnum
                lastTime = user.LastGCTime
                calcInterval = user.UserInfo.GeoEnumInterval
        End Select

        Dim span As TimeSpan = DateTime.Now - lastTime
        Dim waitSecond As Double = Math.Ceiling(calcInterval - span.TotalSeconds)
        If waitSecond > 0 Then
            If showForm Then
                Using form As WaitForm = New WaitForm(waitSecond)
                    If form.ShowDialog(owner) <> DialogResult.OK Then
                        Return False
                    End If
                End Using
            Else
                System.Threading.Thread.Sleep(waitSecond * 1000)
            End If
        End If

        Return True
    End Function

#End Region

    '----------------------------------------------------------------------
    '  public methods
    '----------------------------------------------------------------------
#Region "public methods"

    ''' <summary>
    ''' コンストラクタ
    ''' </summary>
    ''' <param name="waitSeconds">待機秒数</param>
    Sub New(ByVal waitSeconds As Double)
        InitializeComponent()

        Me.waitSeconds_ = waitSeconds
    End Sub

#End Region

    '----------------------------------------------------------------------
    '  private event handlers
    '----------------------------------------------------------------------
#Region "private event handlers"

    ''' <summary>
    ''' フォームロード
    ''' </summary>
    ''' <param name="sender"></param>
    ''' <param name="e"></param>
    Private Sub WaitForm_Load(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MyBase.Load
        progressBar.Minimum = 0
        progressBar.Maximum = Me.waitSeconds_
        progressBar.Value = 0
        labelText.Text = String.Format("計算可能になるまで残り {0} 秒です", Me.waitSeconds_)

        timer.Start()
    End Sub

    ''' <summary>
    ''' タイマー指定時間経過
    ''' </summary>
    ''' <param name="sender"></param>
    ''' <param name="e"></param>
    Private Sub timer_Tick(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles timer.Tick
        Me.currentSeconds_ += 1
        progressBar.Value += 1
        labelText.Text = String.Format("計算可能になるまで残り {0} 秒です", Me.waitSeconds_ - Me.currentSeconds_)

        If Me.currentSeconds_ >= waitSeconds_ Then
            timer.Stop()
            Me.DialogResult = DialogResult.OK
        End If
    End Sub

    ''' <summary>
    ''' キャンセルボタンクリック
    ''' </summary>
    ''' <param name="sender"></param>
    ''' <param name="e"></param>
    Private Sub buttonCancel_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles buttonCancel.Click
        timer.Stop()
    End Sub

#End Region

    '----------------------------------------------------------------------
    '  private fields
    '----------------------------------------------------------------------
#Region "private fields"

    ''' <summary>
    ''' 待機秒数
    ''' </summary>
    Private waitSeconds_ As Integer

    ''' <summary>
    ''' 現在待機した秒数
    ''' </summary>
    Private currentSeconds_ As Integer

#End Region


End Class